<?php

namespace Confident\ApiObjects\Http;

use Confident\Utilities\JsonHelper;

class Response
{
    private $headers = array();
    private $status = 200;
    private $statusCodes = array(
        // Informational 1xx
        100 => 'Continue',101 => 'Switching Protocols',

        // Success 2xx
        200 => 'OK', 201 => 'Created', 202 => 'Accepted',
        203 => 'Non-Authoritative Information',
        204 => 'No Content', 205 => 'Reset Content', 206 => 'Partial Content',

        // Redirection 3xx
        300 => 'Multiple Choices',
        301 => 'Moved Permanently',
        302 => 'Found',  // 1.1
        303 => 'See Other',
        304 => 'Not Modified',
        305 => 'Use Proxy',
        // 306 is deprecated but reserved
        307 => 'Temporary Redirect',

        // Client Error 4xx
        400 => 'Bad Request',
        401 => 'Unauthorized',
        402 => 'Payment Required',
        403 => 'Forbidden',
        404 => 'Not Found',
        405 => 'Method Not Allowed',
        406 => 'Not Acceptable',
        407 => 'Proxy Authentication Required',
        408 => 'Request Timeout',
        409 => 'Conflict',
        410 => 'Gone',
        411 => 'Length Required',
        412 => 'Precondition Failed',
        413 => 'Request Entity Too Large',
        414 => 'Request-URI Too Long',
        415 => 'Unsupported Media Type',
        416 => 'Requested Range Not Satisfiable',
        417 => 'Expectation Failed',

        // Server Error 5xx
        500 => 'Internal Server Error',
        501 => 'Not Implemented',
        502 => 'Bad Gateway',
        503 => 'Service Unavailable',
        504 => 'Gateway Timeout',
        505 => 'HTTP Version Not Supported',
        509 => 'Bandwidth Limit Exceeded'
    );

    public function setHeader($key, $value)
    {
        $this->headers[$key] = $value;
        return $this;
    }

    public function setStatus($status = 200)
    {
        if (is_int($status)) {
            if (isset($this->statusCodes[$status])) {
                $this->status = $status;
                return $this;
            }
            throw new Exception("Status code not exists", 1);
        }
        throw new Exception("Status code should be a integer number", 1);
    }

    private function writeHeaders()
    {
        foreach ($this->headers as $key => $value) {
            header($key.':'.$value);
        }
    }

    private function writeStatus()
    {
        http_response_code($this->status);
    }

    public function write($body = null)
    {
        $this->writeHeaders();
        if (isset($body)) {
            if ($body==='') {
                $body = $this->statusCodes[$this->status];
            }
        }
        echo JsonHelper::Serialize(array('message' => $body));
    }
}
